#include <engine/math.h>

#include <engine/3d/camera.h>
#include <engine/3d/entity.h>

/*
 * camera3d_reg_clear:
 *	Initializes a Camera3D's registry.
 */
void
camera3d_reg_clear(p_camera)
struct Camera3D *p_camera;
{
	p_camera->nrot = 0;
	p_camera->ntran = 0;
}

/*
 * camera3d_reg_tran:
 *	Registers a translation VECTOR to a Camera3D struct.
 */
void
camera3d_reg_tran(p_camera, p_tran)
struct Camera3D *p_camera;
VECTOR *p_tran;
{
	p_camera->ntran++;
	p_camera->p_tran[p_camera->ntran-1] = p_tran;
}

/*
 * camera3d_reg_rot:
 *	Registers a rotation SVECTOR to a Camera3D struct.
 */
void
camera3d_reg_rot(p_camera, p_rot)
struct Camera3D *p_camera;
SVECTOR *p_rot;
{
	p_camera->nrot++;
	p_camera->p_rot[p_camera->nrot-1] = p_rot;
}

/*
 * camera3d_reg_ent:
 *	Registers an Entity3D struct to a Camera3D struct.
 */
void
camera3d_reg_ent(p_camera, p_ent)
struct Camera3D *p_camera;
struct Entity3D *p_ent;
{
	camera3d_reg_rot(p_camera, &p_ent->rotation);
	camera3d_reg_tran(p_camera, &p_ent->translation);
}

/*
 * camera3d_apply_tran:
 *	Applies a translation vector to all registered entities.
 */
void
camera3d_apply_tran(p_camera, p_tran)
struct Camera3D *p_camera;
VECTOR *p_tran;
{
	int i;

	for(i = 0; i < p_camera->ntran; i++)
	{
		setVector(p_camera->p_tran[i], p_tran->vx, p_tran->vy, p_tran->vz);
	}
}

/*
 * camera3d_apply_rot:
 *	Applies angular velocity to camera and subsequent entities.
 *	If camera is locked however, it sets total rotation of camera
 *	and relevant game objects to the p_camera_locked vector.
 */
void
camera3d_apply_rot(p_camera, p_camera_locked)
struct Camera3D *p_camera;
SVECTOR *p_camera_locked;
{
	int i, j;

	if(p_camera->is_locked)
	{
		setVector(&p_camera->rotation, p_camera_locked->vx, p_camera_locked->vy, p_camera_locked->vz);
	}
	else
	{
		p_camera->rotation.vx += p_camera->angular_velocity.vx;
		p_camera->rotation.vy += p_camera->angular_velocity.vy;
	}

	for(i = 0; i < p_camera->nrot; i++)
	{
		if(p_camera->is_locked)
		{
			p_camera->p_rot[i]->vx = p_camera_locked->vx;
			p_camera->p_rot[i]->vy = p_camera_locked->vy;
			p_camera->p_rot[i]->vz = p_camera_locked->vz;
		}
		else
		{
			p_camera->p_rot[i]->vx += p_camera->angular_velocity.vx;
			p_camera->p_rot[i]->vy += p_camera->angular_velocity.vy;
			p_camera->p_rot[i]->vz += p_camera->angular_velocity.vz;
		}
	}
}

/*
 * camera3d_position:
 *	Updates the local and world coords of a Camera3D.
 */
void
camera3d_position(p_camera, p_player, tran_z)
struct Camera3D *p_camera;
SVECTOR *p_player;
int tran_z;
{
	p_camera->local_pos.vx = (((rcos(p_camera->rotation.vx)*rsin(p_camera->rotation.vy))>>12)*tran_z)>>12;
	p_camera->local_pos.vy = (rsin(p_camera->rotation.vx)*tran_z)>>12;
	p_camera->local_pos.vz = -(((rcos(p_camera->rotation.vx)*rcos(p_camera->rotation.vy))>>12)*tran_z)>>12;

	p_camera->global_pos.vx = -(-p_player->vx + p_camera->local_pos.vx);
	p_camera->global_pos.vy = -p_player->vy + p_camera->local_pos.vy + p_camera->offset.vy;
	p_camera->global_pos.vz = -(-p_player->vz + p_camera->local_pos.vz);
}

